/*
 * Source code of the guitar processor.
 * Applies the selected effect on the input signal.
 *
 * @author Gabriel BERCARU (78505), 336CB
 */

#include <avr/interrupt.h>
#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>

#define NUM_EFFECTS 3
#define NUM_DEFAULT_SEQUENCES 3
#define NUM_EFFECTS_IN_SEQUENCE 5

/*
 * 1 - no effect
 * 2 - blue clipper
 * 4 - jordan-bosstone
 */

int effects[3] = {1, 2, 4}; // Effects implemented
int current_effect_index = 0; // Which effect is currently active (useful only if not currently playing a predefined sequence of effects)

int playing_sequence = 0; // Tells if a predefined sequence of effects is currently playing
int sequences[3][5] = {{1, 2, 4, 2, 1}, {2, 1, 4, 1, 2}, {4, 1, 2, 1, 4}}; //Table to keep track of the predefined sequences
int current_sequence_index = 0; // Which sequence is currently playing
int sequence_indexes[3] = {0, 0, 0}; // Keeps track of the active effect inside each predefined sequence

ISR(PCINT1_vect)
{
	if ((PINB & (1 << PB0)) == 0) {
		// Next effect
		current_effect_index = (current_effect_index + 1) % NUM_EFFECTS;
		playing_sequence = 0;
    } else if ((PINB & (1 << PB1)) == 0) {
		// Next sequence of effects
		current_sequence_index = (current_sequence_index + 1) % NUM_DEFAULT_SEQUENCES;
		playing_sequence = 1;
	} else if ((PINB & (1 << PB2)) == 0 && playing_sequence == 1) {
		// Next effect in the current sequence
		sequence_indexes[current_sequence_index] = (sequence_indexes[current_sequence_index] + 1) % NUM_EFFECTS_IN_SEQUENCE;
	}
}

void setup_intr(void)
{
	DDRB &= ~(1 << PB0); // PB0 set as input
	PORTB |= (1 << PB0); // Pull up resistor activated
	
	DDRB &= ~(1 << PB1); // PB1 set as input
	PORTB |= (1 << PB1); // Pull up resistor activated
	
	DDRB &= ~(1 << PB2); // PB2 set as input
	PORTB |= (1 << PB2); // Pull up resistor activated
	
    PCICR |= (1 << PCIE1) | (1 << PCIE3);
	
    PCMSK1 |= (1 << PCINT10);
    PCMSK3 |= (1 << PCINT30);
}

void setup_output(void)
{
	// Set CD4066 enable signals as output
	DDRD |= (1 << PD0);
	DDRD |= (1 << PD1);
	DDRD |= (1 << PD2);
}

int main(void)
{
	sei();
	setup_intr();
	setup_output();
	
	while (1) {
		// Set enable signal for ONLY ONE of the distortion circuits
		// 1 = 001
		// 2 = 010
		// 4 = 100
		// Only one channel will be active at a time
		PORTD = effects[current_effect_index];
	}
}